@@ -168,11 +168,19 @@ extern char *strerror(int);
 #endif
 
 #if defined(MM_SHMT_IPCSHM)
-#define KERNEL 1   /* For SunOS4 */
-#define _KMEMUSER  /* For BS2000 to enable SHM_[RW] */
+#ifdef MM_OS_SUNOS
+#define KERNEL 1
+#endif
+#ifdef MM_OS_BS2000
+#define _KMEMUSER 
+#endif
 #include <sys/shm.h>
+#ifdef MM_OS_SUNOS
 #undef KERNEL
+#endif
+#ifdef MM_OS_BS2000
 #undef _KMEMUSER
+#endif
 #if !defined(SHM_R)
 #define SHM_R 0400
 #endif
