@@ -79,8 +79,8 @@ MM *mm_create(size_t usize, const char *file)
     mm->mp_offset  = SIZEOF_mem_pool;
 
     /* first element of list of free chunks counts existing chunks */
-    mm->mp_freechunks.mc_size      = 0;
-    mm->mp_freechunks.mc_usize     = 0;
+    mm->mp_freechunks.mc_size      = 0; /* has to be 0 forever */
+    mm->mp_freechunks.mc_usize     = 0; /* counts chunks */
     mm->mp_freechunks.mc_u.mc_next = NULL;
 
     return mm;
@@ -196,18 +196,23 @@ static void mm_insert_chunk(MM *mm, mem_chunk *mcInsert)
         mc = mc->mc_u.mc_next;
     mcPrev = mc;
     mcNext = mc->mc_u.mc_next;
-    if ((char *)mcPrev+mcPrev->mc_size == (char *)mcInsert &&
-             (mcNext != NULL && (char *)mcInsert+mcInsert->mc_size == (char *)mcNext)) {
+    if (mcPrev == mcInsert || mcNext == mcInsert) {
+        mm_core_unlock((void *)mm);
+        ERR(MM_ERR_ALLOC, "chunk of memory already in free list");
+        return;
+    }
+    if ((char *)mcPrev+(mcPrev->mc_size) == (char *)mcInsert &&
+        (mcNext != NULL && (char *)mcInsert+(mcInsert->mc_size) == (char *)mcNext)) {
         /* merge with previous and next chunk */
         mcPrev->mc_size += mcInsert->mc_size + mcNext->mc_size;
         mcPrev->mc_u.mc_next = mcNext->mc_u.mc_next;
         mm->mp_freechunks.mc_usize -= 1;
     }
-    else if ((char *)mcPrev+mcPrev->mc_size == (char *)mcInsert) {
+    else if ((char *)mcPrev+(mcPrev->mc_size) == (char *)mcInsert) {
         /* merge with previous chunk */
         mcPrev->mc_size += mcInsert->mc_size;
     }
-    else if (mcNext != NULL && (char *)mcInsert+mcInsert->mc_size == (char *)mcNext) {
+    else if (mcNext != NULL && (char *)mcInsert+(mcInsert->mc_size) == (char *)mcNext) {
         /* merge with next chunk */
         mcInsert->mc_size += mcNext->mc_size;
         mcInsert->mc_u.mc_next = mcNext->mc_u.mc_next;
@@ -238,13 +243,16 @@ static mem_chunk *mm_retrieve_chunk(MM *mm, size_t size)
     size_t sMin;
     size_t s;
 
+    if (size == 0)
+        return NULL;
     if (mm->mp_freechunks.mc_usize == 0)
         return NULL;
     if (!mm_core_lock((void *)mm, MM_LOCK_RW))
         return NULL;
+
     /* find best-fitting chunk */
     pmcMin = NULL;
-    sMin = mm->mp_size; /* maximum possible */
+    sMin = mm->mp_size; /* initialize with maximum possible */
     mc = &(mm->mp_freechunks);
     while (mc->mc_u.mc_next != NULL) {
         s = mc->mc_u.mc_next->mc_size;
@@ -256,6 +264,7 @@ static mem_chunk *mm_retrieve_chunk(MM *mm, size_t size)
         }
         mc = mc->mc_u.mc_next;
     }
+
     /* create result chunk */
     if (pmcMin == NULL)
         mcRes = NULL;
@@ -277,6 +286,7 @@ static mem_chunk *mm_retrieve_chunk(MM *mm, size_t size)
             mm->mp_freechunks.mc_usize--;
         }
     }
+
     mm_core_unlock((void *)mm);
     return mcRes;
 }
@@ -301,7 +311,7 @@ void *mm_malloc(MM *mm, size_t usize)
         return NULL;
     if ((mm->mp_size - mm->mp_offset) < size) {
         mm_core_unlock((void *)mm);
-        ERR(MM_ERR_ALLOC, "Out of memory");
+        ERR(MM_ERR_ALLOC, "out of memory");
         errno = ENOMEM;
         return NULL; 
     }
