@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 1999 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1999-2000 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -104,11 +104,12 @@ char *mm_lib_error_get(void)
  * Version Information
  */
 
-#define _AS_HEADER
+#define _MM_VERS_C_AS_HEADER_
 #include "mm_vers.c"
+#undef  _MM_VERS_C_AS_HEADER_
 
 int mm_lib_version(void)
 {
-    return MM_Version;
+    return MM_VERSION;
 }
 
